var assign, defaultLang, hash, loaderUtils, parse, path, rewriterInjectRE, rewriters, selectorPath, templateLoader;

loaderUtils = require('loader-utils');

assign = require('object-assign');

parse = require('./parser');

selectorPath = require.resolve('./selector');

hash = require('hash-sum');

path = require('path');

defaultLang = {
  template: 'html',
  style: 'css',
  script: 'js'
};

rewriterInjectRE = /\b((css|(vue-)?html)(-loader)?(\?[^!]+)?)(?:!|$)/;

rewriters = {
  template: require.resolve('./template-rewriter'),
  style: require.resolve('./style-rewriter')
};

templateLoader = require.resolve('./template-loader');

module.exports = function(content) {
  var defaultLoaders, fileName, filePath, hasLocalStyles, hotId, loaderContext, loaders, moduleId, options, output, parts, query, script, template, templateWarnings;
  defaultLoaders = {
    html: 'vue-html-loader',
    css: 'vue-style-loader!css-loader',
    js: 'babel-loader?presets[]=es2015&plugins[]=transform-runtime&comments=false'
  };
  this.cacheable();
  loaderContext = this;
  options = this.options.vue || {};
  query = loaderUtils.parseQuery(this.query);
  filePath = this.resourcePath;
  fileName = path.basename(filePath);
  moduleId = '_v-' + hash(filePath);
  if (this.options.babel) {
    defaultLoaders.js = 'babel-loader';
  }
  if (this.sourceMap && !this.minimize && options.cssSourceMap !== false && process.env.NODE_ENV !== 'production') {
    defaultLoaders.css = 'vue-style-loader!css-loader?sourceMap';
  }
  loaders = assign({}, defaultLoaders, options.loaders);
  function getRequire(type, part, index, scoped) {
    return "require(" + (getRequireString(type, part, index, scoped)) + ")\n";
  };
  function getRequireString(type, part, index, scoped) {
    return loaderUtils.stringifyRequest(loaderContext, '!!' + getLoaderString(type, part, scoped) + getSelectorString(type, index || 0) + filePath);
  };
  function getRequireForImport(type, impt, scoped) {
    return "require(" + (getRequireForImportString(type, impt, scoped)) + ")\n";
  };
  function getRequireForImportString(type, impt, scoped) {
    return loaderUtils.stringifyRequest(loaderContext, '!!' + getLoaderString(type, impt, scoped) + impt.src);
  };
  function getLoaderString(type, part, scoped) {
    var injectString, lang, loader, rewriter;
    lang = part.lang || defaultLang[type];
    loader = loaders[lang];
    rewriter = getRewriter(type, scoped);
    injectString = type === 'script' && query.inject ? "inject!" : "";
    if (loader !== void 0) {
      if (rewriterInjectRE.test(loader)) {
        loader = loader.replace(rewriterInjectRE, function(m, $1) {
          return ensureBang($1) + rewriter;
        });
      } else {
        loader = ensureBang(loader) + rewriter;
      }
      return injectString + ensureBang(loader);
    } else {
      switch (type) {
        case 'template':
          return "" + defaultLoaders.html + "!" + rewriter + templateLoader + "?raw&engine=" + lang + "!";
        case 'style':
          return "" + defaultLoaders.css + "!" + rewriter + lang + "!";
        case 'script':
          return "" + injectString + lang + "!";
      }
    }
  };
  function getRewriter(type, scoped) {
    var meta;
    meta = "?id=" + moduleId;
    switch (type) {
      case 'template':
        if (scoped) {
          return "" + rewriters.template + meta + "!";
        } else {
          return "";
        }
      case 'style':
        return rewriters.style + (scoped ? "" + meta + "&scoped=true!" : "!");
      default:
        return "";
    }
  };
  function getSelectorString(type, index) {
    return selectorPath + '?type=' + type + '&index=' + index + '!';
  };
  function ensureBang(loader) {
    if (loader.charAt(loader.length - 1) !== '!') {
      return "" + loader + "!";
    } else {
      return loader;
    }
  };
  parts = parse(content, fileName, this.sourceMap);
  hasLocalStyles = false;
  output = 'var __vue_script__, __vue_template__\n';
  templateWarnings = parts.template.length && parts.template[0].warnings;
  if (templateWarnings) {
    templateWarnings.forEach(this.emitError);
  }
  parts.styleImports.forEach(function(impt) {
    if (impt.scoped) {
      hasLocalStyles = true;
    }
    return output += getRequireForImport('style', impt, impt.scoped);
  });
  parts.style.forEach(function(style, i) {
    if (style.scoped) {
      hasLocalStyles = true;
    }
    return output += getRequire('style', style, i, style.scoped);
  });
  script = void 0;
  if (parts.script.length) {
    script = parts.script[0];
    output += '__vue_script__ = ' + (script.src ? getRequireForImport('script', script, 0) : getRequire('script', script, 0));
    if (!this.minimize) {
      output += 'if (__vue_script__ &&\n' + '    __vue_script__.__esModule &&\n' + '    Object.keys(__vue_script__).length > 1) {\n' + '  console.warn(' + JSON.stringify('[vue-loader] ' + path.relative(process.cwd(), filePath) + ': named exports in *.vue files are ignored.') + ')' + '}\n';
    }
  }
  template = void 0;
  if (parts.template.length) {
    template = parts.template[0];
    output += '__vue_template__ = ' + (template.src ? getRequireForImport('template', template, hasLocalStyles) : getRequire('template', template, 0, hasLocalStyles));
  }
  if (!query.inject) {
    output += 'module.exports = __vue_script__ || {}\n' + 'if (module.exports.__esModule) module.exports = module.exports.default\n' + 'if (__vue_template__) {\n' + '(typeof module.exports === "function" ' + '? (module.exports.options || (module.exports.options = {})) ' + ': module.exports).template = __vue_template__\n' + '}\n';
    if (!this.minimize && process.env.NODE_ENV !== 'production' && (parts.script.length || parts.template.length)) {
      hotId = JSON.stringify(moduleId + '/' + fileName);
      output += 'if (module.hot) {(function () {' + '  module.hot.accept()\n' + '  var hotAPI = require("vue-hot-reload-api")\n' + '  hotAPI.install(require("vue"), false)\n' + '  if (!hotAPI.compatible) return\n' + '  var id = ' + hotId + '\n' + '  if (!module.hot.data) {\n' + '    hotAPI.createRecord(id, module.exports)\n' + '  } else {\n' + '    hotAPI.update(id, module.exports, __vue_template__)\n' + '  }\n' + '})()}';
    }
  } else {
    output += 'module.exports = function (injections) {\n' + '  var mod = __vue_script__\n' + '    ? __vue_script__(injections)\n' + '    : {}\n' + '  if (mod.__esModule) mod = mod.default\n' + '  if (__vue_template__) { (typeof mod === "function" ? mod.options : mod).template = __vue_template__ }\n' + '  return mod\n' + '}';
  }
  return output;
};

module.exports.withLoaders = function() {
  throw new Error('vue.withLoaders has been deprecated in vue-loader 6.0. ' + 'Add a "vue" section to the webpack config instead.');
};
