var addId, assign, loaderUtils, postcss, selectorParser;

postcss = require('postcss');

selectorParser = require('postcss-selector-parser');

loaderUtils = require('loader-utils');

assign = require('object-assign');

addId = postcss.plugin('add-id', function(opts) {
  return function(root) {
    function rewriteSelector(node) {
      if (!node.selector) {
        if (node.type === 'atrule' && node.name === 'media') {
          node.each(rewriteSelector);
        }
        return;
      }
      return node.selector = selectorParser(function(selectors) {
        return selectors.each(function(selector) {
          var node2;
          node2 = null;
          selector.each(function(n) {
            if (n.type !== 'pseudo') {
              return node2 = n;
            }
          });
          return selector.insertAfter(node2, selectorParser.attribute({
            attribute: opts.id
          }));
        });
      }).process(node.selector).result;
    };
    return root.each(rewriteSelector);
  };
});

function isObject(val) {
  return val && typeof val === 'object';
};

module.exports = function(css, map) {
  var autoprefixOptions, autoprefixer, cb, file, option, options, opts, plugins, postcssOptions, query;
  this.cacheable();
  cb = this.async();
  query = loaderUtils.parseQuery(this.query);
  options = this.options.vue || {};
  autoprefixOptions = options.autoprefixer;
  postcssOptions = options.postcss;
  plugins = void 0;
  if (Array.isArray(postcssOptions)) {
    plugins = postcssOptions;
  } else if (typeof postcssOptions === 'function') {
    plugins = postcssOptions.call(this, this);
  } else if (isObject(postcssOptions) && postcssOptions.plugins) {
    plugins = postcssOptions.plugins;
  }
  plugins = plugins ? plugins.slice() : [];
  if (query.scoped) {
    plugins.push(addId({
      id: query.id
    }));
  }
  if (autoprefixOptions !== false) {
    autoprefixOptions = assign({}, this.options.autoprefixer, autoprefixOptions);
    autoprefixer = require('autoprefixer')(autoprefixOptions);
    plugins.push(autoprefixer);
  }
  file = this.resourcePath;
  opts = {
    from: file,
    to: file,
    map: false
  };
  if (this.sourceMap && !this.minimize && options.cssSourceMap !== false && process.env.NODE_ENV !== 'production' && !(isObject(postcssOptions) && postcssOptions.options && postcssOptions.map)) {
    opts.map = {
      inline: false,
      annotation: false,
      prev: map
    };
  }
  if (isObject(postcssOptions) && postcssOptions.options) {
    for (option in postcssOptions.options) {
      if (!opts.hasOwnProperty(option)) {
        opts[option] = postcssOptions.options[option];
      }
    }
  }
  return postcss(plugins).process(css, opts).then(function(result) {
    var m;
    m = result.map && result.map.toJSON();
    cb(null, result.css, m);
    return;
  })["catch"](function(e) {
    console.log(e);
    return cb(e);
  });
};
